/******************************************************************************
* File Name: main_cm4.c
*
* Version: 1.0
*
* Description: This example shows how to re-direct the printf function to 
* use UART API to send data to external terminal.
*
* Related Document: CE223001_PSoC6MCU_UART_printf.pdf
*
* Hardware Dependency: CY8CKIT-062-BLE PSoC 6 BLE Pioneer kit
*
******************************************************************************
* Copyright (2018), Cypress Semiconductor Corporation.
******************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("ACTIVE Risk Product"). By including Cypress's 
* product in a ACTIVE Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*****************************************************************************/
#include "project.h"
#include <stdio.h>

#define MAX_COUNT 65535u

/*******************************************************************************
*        Function Prototypes
*******************************************************************************/
static void HandleError(void);
static void SwInterruptIsr(void);


/*******************************************************************************
*        Global variables
*******************************************************************************/
int count = 0u;
int ucount =0;
uint32_t printCount = false;
uint8_t u8RxStatue=0;
uint8_t u8RxBuf[8];
uint8_t u8RxCnt;
uint8_t u8Crc;

uint32_t read_data;
uint32_t data_received;
uint8_t uart_error;
/*******************************************************************************
* Function Name: SwInterruptIsr
********************************************************************************
* Summary: 
* This function is executed when GPIO interrupt is triggered.
* This ISR 
*        i. Clears the interrupt source.
*        ii. Increments the count and sets printCount flag.
*       
* \Param None
*
* \return
* void
*
*******************************************************************************/

uint8 test_my=0;
static void SwInterruptIsr(void)
{   
    test_my=0x55;
    
   printf("Entering Deep Sleep = %d\r\n", count);
    Cy_GPIO_ClearInterrupt(Pin_Switch_0_PORT, Pin_Switch_0_NUM);
        
    count++;
    /* If count reaches MAX_COUNT reset the count variable. */
    if(count == MAX_COUNT)
    {
        count = 0u;   
    }
    printCount = true;
    
	NVIC_ClearPendingIRQ(SysInt_SW_cfg.intrSrc);
}

static void SysInt_UART_Rx_Interrupt(void)
{
   Cy_GPIO_ClearInterrupt(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM);
   ucount++;
   
}
/***************************************************************************//**
* Function Name: ISR_UART
********************************************************************************
*
* Summary:
*  This function is registered to be called when UART interrupt occurs.
*  (Note that only RX fifo not empty interrupt is unmasked)
*  Whenever there is a data in UART RX fifo, Get that data and Put it into 
*  UART TX fifo which will be transmitted to terminal
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void ISR_UART(void)
{
    /* Check for "RX fifo not empty interrupt" */
    if((Uart_Printf_HW->INTR_RX_MASKED & SCB_INTR_RX_MASKED_NOT_EMPTY_Msk ) != 0)
	{
   
        
        /* Clear UART "RX fifo not empty interrupt" */
		Uart_Printf_HW->INTR_RX = Uart_Printf_HW->INTR_RX & SCB_INTR_RX_NOT_EMPTY_Msk;        
            
        /* Get the character from terminal */
		read_data = Cy_SCB_UART_Get(Uart_Printf_HW);
          while (0UL == Cy_SCB_UART_Put(Uart_Printf_HW,read_data))
            {
            }
         printf("uart receive correct = %d\r\n", count);
        
        /* Update data_received flag */
        data_received = 1;        
	}   
    else
    {
        /* Error if any other interrupt occurs */
        uart_error = 1;
       
    }
}

/*******************************************************************************
* Function Name: main
****************************************************************************//**
*
* Summary:
*  The main function performs the following actions:
*   1. Sets up UART component.
*   2. If initialization of UART component fails then stay in an infinite loop.
*   3. Continuously check whether printCount flag is set.
*   4. If printCount flag is set, reset this flag and print the count using
*      printf function.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/

void Process_Deepsleep_to_WAKE_UP(void);


int main(void)
{
     /* UART initialization status */
    cy_en_scb_uart_status_t uart_status ;
    uint8_t u8RxData;
    
     /* Initialize UART operation. Config and Context structure is copied from Generated source. */
    uart_status  = Cy_SCB_UART_Init(Uart_Printf_HW, &Uart_Printf_config, &Uart_Printf_context);
    if(uart_status != CY_SCB_UART_SUCCESS)
    {
        HandleError();
    }	
    Cy_SCB_UART_Enable(Uart_Printf_HW);
    
    /* Initialize and enable GPIO interrupt assigned to CM4+ */
    /* SysInt_SW_cfg structure is defined in cyfitter_sysint_cfg.c based on
       parameters entered in the Interrupts tab of CYDWR. */
    Cy_SysInt_Init(&SysInt_SW_cfg, SwInterruptIsr); 
    NVIC_ClearPendingIRQ(SysInt_SW_cfg.intrSrc);
    NVIC_EnableIRQ((IRQn_Type)SysInt_SW_cfg.intrSrc);
    
    Cy_SysInt_Init(&SysInt_UART_Rx_cfg, SysInt_UART_Rx_Interrupt); 

     __enable_irq(); /* Enable global interrupts. */
    
   // printf("Number of times switch SW2 has been pressed = %d\r\n", count);
    for(;;)
    {
    
        CyDelay(100);
   
    /* Unmasking only the RX fifo not empty interrupt bit */
    Uart_Printf_HW->INTR_RX_MASK = SCB_INTR_RX_MASK_NOT_EMPTY_Msk;
    
    /* Interrupt Settings for UART */    
    Cy_SysInt_Init(&Uart_Printf_SCB_IRQ_cfg, ISR_UART);
    
    /* Enable the interrupt */
    NVIC_EnableIRQ(Uart_Printf_SCB_IRQ_cfg.intrSrc);
    
    /* Initialize flags */
    data_received = 0;
    uart_error = 0;
      __enable_irq(); 
        for(;;)
    {
        if(test_my==0x55)
         {
            
            CyDelay(100);//waing for the uart finished to transfer the data
            test_my=0x00; 
            Process_Deepsleep_to_WAKE_UP();  

        }
        

    }

      
    }
}


void Process_Deepsleep_to_WAKE_UP()
{

        
        Cy_GPIO_SetHSIOM(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, 0x00);
        Cy_GPIO_SetDrivemode(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, CY_GPIO_DM_PULLUP);
       
        Cy_GPIO_SetInterruptEdge(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, CY_GPIO_INTR_FALLING);
        Cy_GPIO_SetInterruptMask(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, 1);
        Cy_GPIO_SetVtrip(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, 0); 
        NVIC_ClearPendingIRQ(SysInt_UART_Rx_cfg.intrSrc);
        NVIC_EnableIRQ((IRQn_Type)SysInt_UART_Rx_cfg.intrSrc);
  
        Cy_SysPm_DeepSleep(CY_SYSPM_WAIT_FOR_INTERRUPT);
     
        Cy_SCB_UART_ClearRxFifo(Uart_Printf_HW);
        Cy_SCB_UART_ClearTxFifo(Uart_Printf_HW);
        CyDelay(1000);  
        
  
         Cy_SCB_UART_ClearTxFifo(Uart_Printf_HW);
       
         printf("wake up success=%d\r\n", ucount);
         printf("wake up success=%d\r\n", ucount); 
        
     
        Cy_GPIO_SetInterruptMask(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, 0);//intrMask
        Cy_GPIO_SetInterruptEdge(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, CY_GPIO_INTR_DISABLE);//intrCfg
        NVIC_DisableIRQ((IRQn_Type)SysInt_UART_Rx_cfg.intrSrc);
        
        #if 1
        Cy_GPIO_SetDrivemode(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, CY_GPIO_DM_HIGHZ);//
        Cy_GPIO_SetHSIOM(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, P5_0_SCB5_UART_RX);
        Cy_GPIO_SetVtrip(Uart_Printf_rx_0_PORT, Uart_Printf_rx_0_NUM, 0);
        Cy_SCB_UART_Enable(Uart_Printf_HW);
        #endif
  
        Uart_Printf_HW->INTR_RX_MASK = SCB_INTR_RX_MASK_NOT_EMPTY_Msk;
        
        /* Interrupt Settings for UART */    
        Cy_SysInt_Init(&Uart_Printf_SCB_IRQ_cfg, ISR_UART);
        
        /* Enable the interrupt */
        NVIC_EnableIRQ(Uart_Printf_SCB_IRQ_cfg.intrSrc);

}



/*******************************************************************************
* Function Name: HandleError
********************************************************************************
* Summary:
* This function processes unrecoverable errors such as UART component 
* initialization error. In case of such error the system will stay in an
* infinite loop of this function.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void HandleError(void)
{
     /* Disable all interrupts */
    __disable_irq();

    while(1u) 
    {
        
    }
}


/*******************************************************************************
* When printf function is called it is redirected to the following functions
* depending on compiler used.
*******************************************************************************/
#if defined (__GNUC__)
/*******************************************************************************
* Function Name: _write
********************************************************************************
* Summary: 
* NewLib C library is used to retarget printf to _write. printf is redirected to 
* this function when GCC compiler is used to print data to terminal using UART. 
*
* \param file
* This variable is not used.
* \param *ptr
* Pointer to the data which will be transfered through UART.
* \param len
* Length of the data to be transfered through UART.
*
* \return
* returns the number of characters transferred using UART.
* \ref int
*******************************************************************************/
    int _write(int file, char *ptr, int len)
    {
        int nChars = 0;

        /* Suppress the compiler warning about an unused variable. */
        if (0 != file)
        {
        }
        
        for (/* Empty */; len != 0; --len)
        {
            /* Block until there is space in the TX FIFO or buffer. */
            while (0UL == Cy_SCB_UART_Put(Uart_Printf_HW, *ptr))
            {
            }
            
            ++nChars;
            ++ptr;
        }
        
        return (nChars);
    }
#elif defined(__ARMCC_VERSION)
    
/*******************************************************************************
* Function Name: fputc
********************************************************************************
* Summary: 
* printf is redirected to this function when MDK compiler is used to print data
* to terminal using UART.
*
* \param ch
* Character to be printed through UART.
*
* \param *file
* pointer to a FILE object that identifies the stream where the character is to be
* written.
*
* \return
* This function returns the character that is written in case of successful
* write operation else in case of error EOF is returned.
* \ref int
*******************************************************************************/
    struct __FILE
    {
        int handle;
    };
    
    enum
    {
        STDIN_HANDLE,
        STDOUT_HANDLE,
        STDERR_HANDLE
    };
    
    FILE __stdin = {STDIN_HANDLE};
    FILE __stdout = {STDOUT_HANDLE};
    FILE __stderr = {STDERR_HANDLE};
    
    int fputc(int ch, FILE *file)
    {
        int ret = EOF;
        switch(file->handle)
        {
            case STDOUT_HANDLE:
                while (0UL == Cy_SCB_UART_Put(Uart_Printf_HW, ch))
                {
                }
                ret = ch;
            break;
                
            case STDERR_HANDLE:
                ret = ch;
            break;
                
            default:
                file = file;
            break;
        }
        return(ret);
    }
    

#endif /* (__GNUC__) */


/* [] END OF FILE */



